# coding: utf-8

import time
import requests
from django.conf import settings
from django.db import models
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _

from jmb.address.models import AddressCoordinates
from jmb.core.db.models import DateModel, UserModel, StatusModel

from image_cropping import ImageRatioField
from easy_thumbnails.files import get_thumbnailer


class Distributor(DateModel, UserModel, StatusModel):
    name = models.CharField(
        max_length=250,
        verbose_name=_("Name"),
        unique=True,
    )
    marker = models.ImageField(
        upload_to='markers',
        blank=True, null=True,
        verbose_name=_("marker")
    )
    cropping = ImageRatioField(
        'marker', '30x30', size_warning=True
    )

    def get_admin_marker(self):
        if self.marker:
            marker_url = get_thumbnailer(self.marker).get_thumbnail({
                'size': (30, 30),
                'box': self.cropping,
                'crop': True,
                'detail': True,
            }).url
            return u'<img src="{}">'.format(marker_url)
        return u""
    get_admin_marker.short_description = _("Marker")
    get_admin_marker.allow_tags = True

    class Meta(object):
        verbose_name = _('Distributor')
        verbose_name_plural = _('Distributors')
        ordering = ("name",)

    def __unicode__(self):
        return u"{0}".format(self.name)


class Branch(AddressCoordinates, DateModel, UserModel, StatusModel):
    distributor = models.ForeignKey(
        Distributor,
        related_name='branches',
        verbose_name=_("Distributor"),
    )
    name = models.CharField(
        max_length=250,
        verbose_name=_("Name"),
        blank=True, null=True,
    )
    city = models.CharField(
        max_length=100,
        verbose_name=_("City"),
    )
    address = models.CharField(
        max_length=250,
        verbose_name=_("Address"),
    )
    cap = models.CharField(
        max_length=5,
        verbose_name=_("Cap"),
    )
    province = models.CharField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_("Province"),
    )
    email = models.EmailField(
        blank=True, null=True,
        verbose_name=_("Email"),
    )
    phone = models.CharField(
        max_length=20,
        blank=True, null=True,
        verbose_name=_("Phone"),
    )
    is_changed = models.BooleanField(
        verbose_name=_("Changed?"), default=False,
    )
    geocoding_fields = ['address', 'city', 'cap']

    class Meta(object):
        verbose_name = _('Branch')
        verbose_name_plural = _('Branches')
        ordering  = ("distributor", "city", "address")

    def __unicode__(self):
        return u"{0}, {1} {2}".format(self.address, self.city, self.cap)

    @property
    def marker_html(self):
        return u"""{name}{address}, {city} {cap} {province}.{email} {phone}""".format(
            name="<h4>{}</h4>".format(self.name) if self.name else "",
            address=self.address,
            city=self.city,
            cap=self.cap,
            province="({})".format(self.province) if self.province else "",
            email="<br>Email: {}".format(self.email) if self.email else "",
            phone="<br>Telefono: {}".format(self.phone) if self.phone else "",
        )

    @property
    def distributor_marker(self):
        if self.distributor.marker:
            thumbnail_url = get_thumbnailer(self.distributor.marker).get_thumbnail({
                'size': (30, 30),
                'box': self.distributor.cropping,
                'crop': True,
                'detail': True,
            }).url
            return thumbnail_url
        return None

    def save(self, *args, **kwargs):
        if self.id:
            old_instance = Branch.objects.get(id=self.id)
            if self.lat != old_instance.lat or self.lng != old_instance.lng:
                self.is_changed = True
        super(Branch, self).save(*args, **kwargs)


if settings.CMS_ENABLED:
    import acomea.distributor.cms.models